==> v0.94.x.tar.gz: opentelemetry-collector-contrib/exporter/otelarrowexporter/config.go <==
// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

package otelarrowexporter // import "github.com/open-telemetry/opentelemetry-collector-contrib/exporter/otelarrowexporter"

import (
	"fmt"
	"time"

	"github.com/open-telemetry/otel-arrow/collector/compression/zstd"
	"github.com/open-telemetry/otel-arrow/pkg/config"
	"go.opentelemetry.io/collector/component"
	"go.opentelemetry.io/collector/config/configcompression"
	"go.opentelemetry.io/collector/config/configgrpc"
	"go.opentelemetry.io/collector/config/configretry"
	"go.opentelemetry.io/collector/exporter/exporterhelper"
	"google.golang.org/grpc"
)

// Config defines configuration for OTLP exporter.
type Config struct {
	// Timeout, Retry, Queue, and gRPC client settings are
	// inherited from exporterhelper using field names
	// intentionally identical to the core OTLP exporter.

	exporterhelper.TimeoutSettings `mapstructure:",squash"` // squash ensures fields are correctly decoded in embedded struct.
	exporterhelper.QueueSettings   `mapstructure:"sending_queue"`

	RetrySettings configretry.BackOffConfig `mapstructure:"retry_on_failure"`

	configgrpc.ClientConfig `mapstructure:",squash"` // squash ensures fields are correctly decoded in embedded struct.

	// Arrow includes settings specific to OTel Arrow.
	Arrow ArrowSettings `mapstructure:"arrow"`

	// UserDialOptions cannot be configured via `mapstructure`
	// schemes.  This is useful for custom purposes where the
	// exporter is built and configured via code instead of yaml.
	// Uses include custom dialer, custom user-agent, etc.
	UserDialOptions []grpc.DialOption `mapstructure:"-"`
}

// ArrowSettings includes whether Arrow is enabled and the number of
// concurrent Arrow streams.
type ArrowSettings struct {
	// NumStreams determines the number of OTel Arrow streams.
	NumStreams int `mapstructure:"num_streams"`

	// MaxStreamLifetime should be set to less than the value of
	// grpc: keepalive: max_connection_age_grace plus the timeout.
	MaxStreamLifetime time.Duration `mapstructure:"max_stream_lifetime"`

	// Zstd settings apply to OTel-Arrow use of gRPC specifically.
	// Note that when multiple Otel-Arrow exporters are configured
	// their settings will be applied in arbitrary order.
	// Identical Zstd settings are recommended when multiple
	// OTel-Arrow exporters are in use.
	Zstd zstd.EncoderConfig `mapstructure:"zstd"`

	// PayloadCompression is applied on the Arrow IPC stream
	// internally and may have different results from using
	// gRPC-level compression.  This is disabled by default, since
	// gRPC-level compression is enabled by default.  This can be
	// set to "zstd" to turn on Arrow-Zstd compression.
	// Note that `Zstd` applies to gRPC, not Arrow compression.
	PayloadCompression configcompression.CompressionType `mapstructure:"payload_compression"`

	// Disabled prevents using OTel Arrow streams.  The exporter
	// falls back to standard OTLP.
	Disabled bool `mapstructure:"disabled"`

	// DisableDowngrade prevents this exporter from fallback back
	// to standard OTLP.  If the Arrow service is unavailable, it
	// will retry and/or fail.
	DisableDowngrade bool `mapstructure:"disable_downgrade"`
}

var _ component.Config = (*Config)(nil)

// Validate checks if the exporter configuration is valid
func (cfg *Config) Validate() error {
	if err := cfg.QueueSettings.Validate(); err != nil {
		return fmt.Errorf("queue settings has invalid configuration: %w", err)
	}
	if err := cfg.Arrow.Validate(); err != nil {
		return fmt.Errorf("arrow settings has invalid configuration: %w", err)
	}

	return nil
}

// Validate returns an error when the number of streams is less than 1.
func (cfg *ArrowSettings) Validate() error {
	if cfg.NumStreams < 1 {
		return fmt.Errorf("stream count must be > 0: %d", cfg.NumStreams)
	}

	if cfg.MaxStreamLifetime.Seconds() < 1 {
		return fmt.Errorf("max stream life must be >= 1s: %d", cfg.MaxStreamLifetime)
	}

	if err := cfg.Zstd.Validate(); err != nil {
		return fmt.Errorf("zstd encoder: invalid configuration: %w", err)
	}

	// The cfg.PayloadCompression field is validated by the underlying library,
	// but we only support Zstd or none.
	switch cfg.PayloadCompression {
	case "none", "", configcompression.Zstd:
	default:
		return fmt.Errorf("unsupported payload compression: %s", cfg.PayloadCompression)
	}
	return nil
}

func (cfg *ArrowSettings) toArrowProducerOptions() (arrowOpts []config.Option) {
	switch cfg.PayloadCompression {
	case configcompression.Zstd:
		arrowOpts = append(arrowOpts, config.WithZstd())
	case "none", "":
		arrowOpts = append(arrowOpts, config.WithNoZstd())
	default:
		// Should have failed in validate, nothing we can do.
	}
	return
}

==> v0.96.x.tar.gz: opentelemetry-collector-contrib/exporter/otelarrowexporter/config.go <==
// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

package otelarrowexporter // import "github.com/open-telemetry/opentelemetry-collector-contrib/exporter/otelarrowexporter"

import (
	"fmt"
	"time"

	"github.com/open-telemetry/otel-arrow/collector/compression/zstd"
	"github.com/open-telemetry/otel-arrow/pkg/config"
	"go.opentelemetry.io/collector/component"
	"go.opentelemetry.io/collector/config/configcompression"
	"go.opentelemetry.io/collector/config/configgrpc"
	"go.opentelemetry.io/collector/config/configretry"
	"go.opentelemetry.io/collector/exporter/exporterhelper"
	"google.golang.org/grpc"
)

// Config defines configuration for OTLP exporter.
type Config struct {
	// Timeout, Retry, Queue, and gRPC client settings are
	// inherited from exporterhelper using field names
	// intentionally identical to the core OTLP exporter.

	exporterhelper.TimeoutSettings `mapstructure:",squash"` // squash ensures fields are correctly decoded in embedded struct.
	exporterhelper.QueueSettings   `mapstructure:"sending_queue"`

	RetrySettings configretry.BackOffConfig `mapstructure:"retry_on_failure"`

	configgrpc.ClientConfig `mapstructure:",squash"` // squash ensures fields are correctly decoded in embedded struct.

	// Arrow includes settings specific to OTel Arrow.
	Arrow ArrowSettings `mapstructure:"arrow"`

	// UserDialOptions cannot be configured via `mapstructure`
	// schemes.  This is useful for custom purposes where the
	// exporter is built and configured via code instead of yaml.
	// Uses include custom dialer, custom user-agent, etc.
	UserDialOptions []grpc.DialOption `mapstructure:"-"`
}

// ArrowSettings includes whether Arrow is enabled and the number of
// concurrent Arrow streams.
type ArrowSettings struct {
	// NumStreams determines the number of OTel Arrow streams.
	NumStreams int `mapstructure:"num_streams"`

	// MaxStreamLifetime should be set to less than the value of
	// grpc: keepalive: max_connection_age_grace plus the timeout.
	MaxStreamLifetime time.Duration `mapstructure:"max_stream_lifetime"`

	// Zstd settings apply to OTel-Arrow use of gRPC specifically.
	// Note that when multiple Otel-Arrow exporters are configured
	// their settings will be applied in arbitrary order.
	// Identical Zstd settings are recommended when multiple
	// OTel-Arrow exporters are in use.
	Zstd zstd.EncoderConfig `mapstructure:"zstd"`

	// PayloadCompression is applied on the Arrow IPC stream
	// internally and may have different results from using
	// gRPC-level compression.  This is disabled by default, since
	// gRPC-level compression is enabled by default.  This can be
	// set to "zstd" to turn on Arrow-Zstd compression.
	// Note that `Zstd` applies to gRPC, not Arrow compression.
	PayloadCompression configcompression.Type `mapstructure:"payload_compression"`

	// Disabled prevents using OTel Arrow streams.  The exporter
	// falls back to standard OTLP.
	Disabled bool `mapstructure:"disabled"`

	// DisableDowngrade prevents this exporter from fallback back
	// to standard OTLP.  If the Arrow service is unavailable, it
	// will retry and/or fail.
	DisableDowngrade bool `mapstructure:"disable_downgrade"`
}

var _ component.Config = (*Config)(nil)

// Validate checks if the exporter configuration is valid
func (cfg *Config) Validate() error {
	if err := cfg.QueueSettings.Validate(); err != nil {
		return fmt.Errorf("queue settings has invalid configuration: %w", err)
	}
	if err := cfg.Arrow.Validate(); err != nil {
		return fmt.Errorf("arrow settings has invalid configuration: %w", err)
	}

	return nil
}

// Validate returns an error when the number of streams is less than 1.
func (cfg *ArrowSettings) Validate() error {
	if cfg.NumStreams < 1 {
		return fmt.Errorf("stream count must be > 0: %d", cfg.NumStreams)
	}

	if cfg.MaxStreamLifetime.Seconds() < 1 {
		return fmt.Errorf("max stream life must be >= 1s: %d", cfg.MaxStreamLifetime)
	}

	if err := cfg.Zstd.Validate(); err != nil {
		return fmt.Errorf("zstd encoder: invalid configuration: %w", err)
	}

	// The cfg.PayloadCompression field is validated by the underlying library,
	// but we only support Zstd or none.
	switch cfg.PayloadCompression {
	case "none", "", configcompression.TypeZstd:
	default:
		return fmt.Errorf("unsupported payload compression: %s", cfg.PayloadCompression)
	}
	return nil
}

func (cfg *ArrowSettings) toArrowProducerOptions() (arrowOpts []config.Option) {
	switch cfg.PayloadCompression {
	case configcompression.TypeZstd:
		arrowOpts = append(arrowOpts, config.WithZstd())
	case "none", "":
		arrowOpts = append(arrowOpts, config.WithNoZstd())
	default:
		// Should have failed in validate, nothing we can do.
	}
	return
}
